/* (C) 2022 YiRing, Inc. */
package com.yiring.common.domain;

import com.yiring.common.snowflake.SnowflakeId;
import jakarta.persistence.*;
import java.time.LocalDateTime;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import org.hibernate.annotations.Comment;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.UpdateTimestamp;
import org.springframework.data.jpa.domain.support.AuditingEntityListener;

/**
 * 基础表抽象类
 *
 * @author Jim
 * @version 0.1
 * 2022/4/17 12:11
 */

@Getter
@Setter
@ToString
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@SuperBuilder(toBuilder = true)
@MappedSuperclass
@EntityListeners(AuditingEntityListener.class)
public abstract class BasicEntity {

    @Comment("主键")
    @Id
    @GeneratedValue(generator = SnowflakeId.GENERATOR)
    @GenericGenerator(name = SnowflakeId.GENERATOR, strategy = SnowflakeId.Strategy.STRING)
    String id;

    @Comment("创建时间")
    @Column(nullable = false)
    @CreationTimestamp
    LocalDateTime createTime;

    @Comment("最后修改时间")
    @Column(nullable = false)
    @UpdateTimestamp
    LocalDateTime updateTime;

    @Comment("删除时间")
    LocalDateTime deleteTime;

    public interface Where {
        String EXIST = " delete_time is null ";

        String DELETE_SET = " set delete_time = now() where id = ? ";
    }
}
